@@ -2,9 +2,46 @@
 #include "conf.h"
 #include "main.h"
 
+GtkTreeView	*accel_tview = NULL;
+GtkListStore	*accel_store = NULL;
+GtkAccelGroup	*accel_group = NULL;
+
 int		debug = 0;
 double          zoom_factor = 1.0;
 
+typedef struct {
+  char	*legend,
+	*widget;
+  gint	 mods;
+  guint	 accel;
+} key_assignment;
+
+key_assignment key_asg[] = {
+  {N_("Zoom in"),	"zoom_in_btn",
+			0,		GDK_KP_Add},
+  {N_("Zoom out"),	"zoom_out_btn",
+			0,		GDK_KP_Subtract},
+  {N_("Fit width"),	"zoom_fit_btn",
+			0,		GDK_w},
+  {N_("Fit page"),	"zoom_width_btn",
+			0,		GDK_p},
+  {N_("Zoom 1:1"),	"zoom_orig_btn",
+			0,		GDK_1},
+  {N_("Next page"),	"next_btn",
+			0,		GDK_Page_Down},
+  {N_("Previous page"),	"back_btn",
+			0,		GDK_Page_Up},
+  {N_("Last page"),	"last_btn",
+			0,		GDK_End},
+  {N_("First page"),	"begin_btn",
+			0,		GDK_Home},
+  {N_("Open file"),	"open_btn",
+			0,		GDK_o},
+  {N_("Quit cbrPager"),	"quit_btn",
+			0,		GDK_q}
+};
+#define nr_keys_assigned (sizeof(key_asg)/sizeof(key_assignment))
+
 preferences pref = {
   ""
 };
@@ -13,6 +50,7 @@ void
 save_config(void)
 {
   char path[120];
+  int ka;
 
   sprintf(path, "%s/global/lastbook", prog_name);
   gnome_config_set_string(path, pref.lastbook);
@@ -47,6 +85,13 @@ save_config(void)
   sprintf(path, "%s/global/nav_hidden", prog_name);
   gnome_config_set_int(path, pref.nav_hidden);
 
+  for (ka = 0; ka < nr_keys_assigned; ka++) {
+    sprintf(path, "%s/accel/%s_mods", prog_name, key_asg[ka].widget);
+    gnome_config_set_int(path, key_asg[ka].mods);
+    sprintf(path, "%s/accel/%s_key", prog_name, key_asg[ka].widget);
+    gnome_config_set_int(path, key_asg[ka].accel);
+  }
+
   gnome_config_sync();
 }
 
@@ -55,7 +100,7 @@ void
 load_config(void)
 {
   char path[120], *p;
-  int def;
+  int def, ka;
 
   sprintf(path, "%s/global/lastbook=", prog_name);
   strcpy(pref.lastbook, p = gnome_config_get_string_with_default(path, &def));
@@ -94,6 +139,16 @@ load_config(void)
 
   if (pref.dirmode == DIRM_FIXED)		// fixed startup directory
     strncpy(pref.lastbook, pref.fixeddir, sizeof(pref.lastbook));
+
+  for (ka = 0; ka < nr_keys_assigned; ka++) {
+    sprintf(path, "%s/accel/%s_mods=%d", prog_name, key_asg[ka].widget,
+		key_asg[ka].mods);
+    key_asg[ka].mods = gnome_config_get_int_with_default(path, &def);
+    sprintf(path, "%s/accel/%s_key=%u", prog_name, key_asg[ka].widget,
+		key_asg[ka].accel);
+    key_asg[ka].accel = gnome_config_get_int_with_default(path, &def);
+  }
+
 }
 
 
@@ -169,6 +224,10 @@ parse_config(GtkWidget *ref)
 {
   GtkWidget *w;
   GtkRequisition req;
+  GtkTreeIter iter;
+  guint accel;
+  gint mods;
+  int ka;
 
   w = lookup_widget(ref, "conf_goto_entry");
   strncpy(pref.fixeddir,
@@ -245,4 +304,113 @@ parse_config(GtkWidget *ref)
 	pref.nav_hor ? GTK_ORIENTATION_HORIZONTAL
 		     : GTK_ORIENTATION_VERTICAL);
   printf("parse: %d\n", pref.nav_hor);
+
+  gtk_tree_model_get_iter_first(GTK_TREE_MODEL(accel_store), &iter);
+  for (ka = 0; ka < nr_keys_assigned; ka++) {
+    gtk_tree_model_get(GTK_TREE_MODEL(accel_store), &iter,
+	ACC_MODS, &mods,
+	ACC_ACCEL, &accel,
+	-1);
+    w = lookup_widget(nav_window, key_asg[ka].widget);
+    gtk_widget_remove_accelerator(w, accel_group,
+		key_asg[ka].accel, key_asg[ka].mods);
+    gtk_widget_add_accelerator(w, "clicked", accel_group,
+		accel, mods, 0);
+    key_asg[ka].mods = mods;
+    key_asg[ka].accel = accel;
+    gtk_tree_model_iter_next(GTK_TREE_MODEL(accel_store), &iter);
+  }
+}
+
+
+void
+conf_accel_tview_cursor_changed(void)
+{
+}
+
+
+void
+accel_edited(GtkCellRendererAccel *renderer, gchar *path, guint accel_key,
+		GdkModifierType mask, guint hardware_keycode,
+		GtkTreeView *treeview)
+{
+  GtkTreeModel *model;
+  GtkTreeIter iter;
+
+  model = gtk_tree_view_get_model(treeview);
+  if (gtk_tree_model_get_iter_from_string(model, &iter, path))
+    gtk_list_store_set (GTK_LIST_STORE(model), &iter,
+		ACC_MODS, (gint) mask,
+		ACC_ACCEL, accel_key,
+		-1);
+}
+
+
+void
+conf_accel_tview_realize(GtkTreeView *tview)
+{
+  GtkCellRenderer *renderer;
+  GtkTreeViewColumn *col;
+  GtkTreeIter iter;
+  int ka;
+
+  accel_tview = tview;
+  accel_store = gtk_list_store_new(ACC_COLS,
+	G_TYPE_STRING,
+	G_TYPE_INT,
+	G_TYPE_UINT);
+
+  gtk_tree_view_set_model(GTK_TREE_VIEW(accel_tview),                             
+        GTK_TREE_MODEL(accel_store));
+  g_object_unref(G_OBJECT(accel_store));
+
+  // Accelerator function
+  renderer = gtk_cell_renderer_text_new();
+  gtk_object_set(GTK_OBJECT(renderer),
+		"ypad", 0, NULL);
+  col = gtk_tree_view_column_new_with_attributes(
+		"Function", renderer,
+		"text", 0, NULL);
+  gtk_tree_view_append_column(GTK_TREE_VIEW(accel_tview), col);
+
+  // Accelerator assignment
+  renderer = gtk_cell_renderer_accel_new();
+  g_object_set(renderer, "accel-mods", GTK_CELL_RENDERER_ACCEL_MODE_GTK,
+		"editable", TRUE, NULL);
+  g_signal_connect(G_OBJECT(renderer), "accel_edited",
+		G_CALLBACK(accel_edited),
+		(gpointer) accel_tview);
+  col = gtk_tree_view_column_new_with_attributes(
+		"Accelerator", renderer,
+		"accel-mods", ACC_MODS,
+		"accel-key", ACC_ACCEL,
+		NULL);
+  gtk_tree_view_append_column(GTK_TREE_VIEW(accel_tview), col);
+  gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(accel_tview), TRUE);
+
+  for (ka = 0; ka < nr_keys_assigned; ka++) {
+    gtk_list_store_append(accel_store, &iter);
+    gtk_list_store_set(accel_store, &iter,
+	ACC_FUNCTION, _(key_asg[ka].legend),
+	ACC_MODS,       key_asg[ka].mods,
+	ACC_ACCEL,      key_asg[ka].accel,
+	-1);
+  }
+}
+
+void
+set_kay_accelerators(void)
+{
+  GtkWidget *w;
+  int ka;
+
+  accel_group = gtk_accel_group_new();
+  gtk_window_add_accel_group(GTK_WINDOW(MainWindow), accel_group);
+
+  for (ka = 0; ka < nr_keys_assigned; ka++) {
+    w = lookup_widget(nav_window, key_asg[ka].widget);
+    gtk_widget_add_accelerator(w, "clicked", accel_group,
+		key_asg[ka].accel,
+		key_asg[ka].mods, 0);
+  }
 }
